/*
	The following C program was used to generate a list of Magic Angles
which are shown in Table 1. These angles are not used as absolute angles
but rather to rotate the unit vectors. (The program was originally done by
Doug Snyder in FORTRAN and was modified by the author.)

	MAGIC ANGLE SEARCH

	Jed Margolin  2001			
*/

#include <math.h>
#include <stdio.h>
#include <conio.h>

double acos(double),sin(double),sqrt(double);

void crunch16(double,int), crunch32(double,int);
//=======================================================================
main()
{
	double limit;

	limit = 0.0005;
	crunch16(limit,10);		// do 10
	printf("\n \n");

	limit = 0.0010;
	crunch16(limit,10);		// do 10
	printf("\n \n");

	limit = 0.0050;
	crunch16(limit,10);		// do 10
	printf("\n \n");

	limit = 0.0005;
	crunch32(limit,20);		// do 20
	printf("\n \n");

	printf("All done\n");
}
//=====================================================================
void crunch16(double limit, int howmany)
{
	double  angle,sine,arcsine,err,mag,magerr;
	int	n,lower,cosine,isine;

	printf("16-bits   0x4000 = 16384\n");
	printf("Magic Angles    Angular Error Limit = %f %% \n",limit);

	printf(" COS       SIN     ISINE   ANGLE(deg)   ");
	printf("%%Error(deg/deg) %%Magnitude Error \n \n ");

	lower = 16000;
	cosine = 16383;

	n = 0;

	while(cosine>lower)
	{

	   //Find Arc Cosine
	   angle = acos(cosine/16384.);

	   // Find corresponding Sine
	   sine = sin(angle)*16384.;	

	   // Round off and convert to integer
	   isine = (sine+0.5);

	/* Find Arc Sine of rounded off value */
		arcsine = asin(isine/16384.);

	/* Calculate percent error */
		err = (arcsine-angle)/angle*100;
		if (err < 0) err = err * -1.;

	/* Length of Vector */
        mag = sqrt((isine/16384.*isine/16384.)+(cosine/16384.*cosine/16384.));
		magerr = (1.0 - mag)*100;

		if (err < limit)

		{
		   printf("%5.0d  ",cosine);
		   printf("%10.3f ",sine);
		   printf("%5.0d  ",isine);
		   printf("%10.8f ",angle*180./3.14159);
		   printf("  %8.7f     ",err);
		   printf("%+8.7f  \n ",magerr);
		   n++;
		}

		cosine = cosine-1;
		if(n>howmany-1)  { return;}
	}
}
//======================================================================
void crunch32(double limit, int howmany)
{
	double  angle,sine,arcsine,err,mag,magerr,maxd;
	int	n,lower,cosine,isine;

	maxd = 0x40000000;

	printf("32-bits   0x40000000 = %.0f \n",maxd);

	printf("Magic Angles    Angular Error Limit = %f %% \n",limit);

	printf(" COS           SIN       ISINE    ANGLE(deg)  ");
	printf("%%Error(deg/deg) %%Magnitude Error\n\n ");

	lower =  0x40000000-100L;	// maximum of 100
	cosine = 0x40000000-1L;

	n = 0;

	while(cosine>lower)
	{

	   //Find Arc Cosine
	   angle = acos(cosine/maxd);

	   // Find corresponding Sine
	   sine = sin(angle)*maxd;	

	   // Round off and convert to integer
	   isine = (sine+0.5);

	/* Find Arc Sine of rounded off value */
		arcsine = asin(isine/maxd);

	/* Calculate percent error */
		err = (arcsine-angle)/angle*100;
		if (err < 0) err = err * -1.;

	/* Length of Vector */
        mag = sqrt((isine/maxd*isine/maxd)+(cosine/maxd*cosine/maxd));
		magerr = (1.0 - mag)*100;

		if (err < limit)

		{
		   printf("%5.0d  ",cosine);
		   printf("%10.3f   ",sine);
		   printf("%5.0d   ",isine);
		   printf("%10.8f ",angle*180./3.14159);
		   printf("  %8.7f      ",err);
		   printf("%+8.7f  \n ",magerr);
		   n++;
		}

		cosine = cosine-1;
		if(n>howmany-1)  { return;}
	}
}
//======================================================================
